package com.persagy.cache;


import com.persagy.entity.AlarmRecordIdsCache;
import com.persagy.repository.AlarmRecordIdsCacheRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * @description: 存放云端返回的报警id, 当报警恢复或过期后移除报警id
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2021/1/21 下午8:28
 * @version: V1.0
 */
@Component
@Slf4j
public class CreatedAlarmIdsCache {
    @Autowired
    private AlarmRecordIdsCacheRepository alarmRecordIdsCacheRepository;
    /**
     * 报警id
     */
    private Set<String> alarmIdSet;

    /**
     * @description: 初始化方法，查询数据库中的数据，放入set
     * @param:
     * @return:
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/21 下午8:43
     * @version: V1.0
     */
    public void init() {
        alarmIdSet = Collections.synchronizedSet(new HashSet<>());
        Iterable<AlarmRecordIdsCache> alarmRecordIds = alarmRecordIdsCacheRepository.findAll();
        alarmRecordIds.forEach(alarmRecordIdsCache -> alarmIdSet.add(alarmRecordIdsCache.getId()));
        log.info("CreatedAlarmIdsCache初始化完成");
        log.info("当前CreatedAlarmIdsCache缓存为：[{}]", alarmIdSet.toString());
    }

    /**
     * @description: 将报警id放入缓存
     * @param: alarmId
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/21 下午8:44
     * @version: V1.0
     */
    public void put(String alarmId) {
        // 存入缓存
        alarmIdSet.add(alarmId);
        // 持久化到数据库
        AlarmRecordIdsCache alarmRecordIdsCache = new AlarmRecordIdsCache();
        alarmRecordIdsCache.setId(alarmId);
        alarmRecordIdsCacheRepository.save(alarmRecordIdsCache);
        log.info("报警id[{}]放入CreatedAlarmIdsCache", alarmId);
        log.info("当前CreatedAlarmIdsCache缓存为：[{}]", alarmIdSet.toString());
    }

    /**
     * @description: 将报警id从缓存中移除
     * @param: alarmId
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/21 下午8:44
     * @version: V1.0
     */
    public void remove(String alarmId) {
        // 从缓存中移除
        alarmIdSet.remove(alarmId);
        // 从数据库中移除
        alarmRecordIdsCacheRepository.deleteById(alarmId);
        log.info("报警id[{}]从CreatedAlarmIdsCache移除", alarmId);
        log.info("当前CreatedAlarmIdsCache缓存为：[{}]", alarmIdSet.toString());
    }

    public boolean contains(String alarmId) {
        return alarmIdSet.contains(alarmId);
    }

    //    public static void main(String[] args) {
    //        HashSet<Integer> objects = new HashSet<>();
    //        objects.add(1);
    //        objects.add(2);
    //        System.out.println(objects.toString());
    //    }
}
