package com.persagy.client;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.TimeInterval;
import com.alibaba.fastjson.JSONObject;
import com.persagy.cache.CreatedAlarmIdsCache;
import com.persagy.entity.NettyMessage;
import com.persagy.enumeration.NettyMsgTypeEnum;
import com.persagy.job.NettyMessageQueue;
import com.persagy.service.impl.NettyMsgHandler;
import com.persagy.utils.StringUtil;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;

import java.util.Arrays;
import java.util.concurrent.TimeUnit;

/**
 * @author lqshi
 * @ClassName: EchoClientHandler
 * @Description: 客户端处理类(处理云端数据)
 * @date 2020年8月29日 下午8:31:59
 */
// 注意：SimpleChannelInboundHandler<ByteBuf>的<>中是什么，channelRead0第二参数是什么
@Slf4j
public class GroupNettyClientHandler extends ChannelInboundHandlerAdapter {
    // Sleep 5 seconds before a reconnection attempt.
    static final int RECONNECT_DELAY = Integer.parseInt(System.getProperty("reconnectDelay", "5"));
    // Reconnect when the server sends nothing for 10 seconds.
    private static final int READ_TIMEOUT = Integer.parseInt(System.getProperty("readTimeout", "10"));

    private GroupNettyClient groupNettyClient;
    private CreatedAlarmIdsCache createdAlarmIdsCache;
    private NettyMsgHandler nettyMsgHandler;


    public GroupNettyClientHandler(
            GroupNettyClient groupNettyClient,
            NettyMsgHandler nettyMsgHandler,
            CreatedAlarmIdsCache createdAlarmIdsCache
    ) {
        this.groupNettyClient = groupNettyClient;
        this.createdAlarmIdsCache = createdAlarmIdsCache;
        this.nettyMsgHandler = nettyMsgHandler;
    }

    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) {

        if (!(evt instanceof IdleStateEvent)) {
            return;
        }
        IdleStateEvent e = (IdleStateEvent) evt;
        if (e.state() == IdleState.READER_IDLE) {
            log.info("no inbound traffic");
            // The connection was OK but there was no traffic for last period.
            // 长时间不操作的时候自动关闭连接; ctx.close();
        }
    }

    /**
     * 在到服务器的连接已经建立之后将被调用
     *
     * @param ctx
     * @throws Exception
     */
    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        log.info("Connected to: " + ctx.channel().remoteAddress());
        ctx.channel().writeAndFlush(new NettyMessage<>(
                NettyMsgTypeEnum.CONNECT, groupNettyClient.projectId).toString());
        //启动的时候发送消息,获取全部报警定义
        //{"groupCode":"wd", "projectId":"Pj123"}
        NettyMessage nettyMessage = new NettyMessage(
                NettyMsgTypeEnum.REQUEST_ALL_CONFIGS, groupNettyClient.projectId);
        nettyMessage.setRemark("连接已经建立；");
        JSONObject content = new JSONObject();
        content.put("groupCode", groupNettyClient.groupCode);
        content.put("projectId", groupNettyClient.projectId);
        nettyMessage.setContent(Arrays.asList(content));
        log.info(nettyMessage.toString());
        ctx.channel().writeAndFlush(nettyMessage.toString());
        iniAlarmResult(ctx);
    }

    private void iniAlarmResult(ChannelHandlerContext ctx) {
        try {
            log.info("--initAlarmResult--");
            TimeInterval timer = DateUtil.timer();
            while (NettyMessageQueue.getNettyMessageQueue().size() > 0 && timer.interval() < 10000) {
                String msg = NettyMessageQueue.getNettyMessageQueue().consume();
                log.info("剩余报警消息令总数:{}", NettyMessageQueue.getNettyMessageQueue().size());
                ctx.writeAndFlush(msg);
            }
        } catch (Exception e) {
            log.error("发送报警消息失败", e);
        }
    }

    /**
     * 当从服务器接收到一个消息时被调用
     *
     * @param ctx
     * @param msg
     * @throws Exception
     */
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        if (StringUtil.isJSONObject((String) msg)) {
            NettyMessage message = StringUtil.tranferItemToDTO((String) msg, NettyMessage.class);
            if (message.getOpCode() == NettyMsgTypeEnum.ALL_CONDITIONS) {
                log.info("获取到全量的报警条件, 共[{}]条", message.getContent().size());
            } else if (message.getOpCode() == NettyMsgTypeEnum.ALL_OBJ_CONDITION_REL) {
                log.info("获取到设备与报警条件的关联关系, 共[{}]条", message.getContent().size());
            } else {
                log.info("接收到netty消息: {}", msg);
            }
        }

        try {
            TimeInterval timer = DateUtil.timer();
            handlerMsg(ctx, msg);
            log.debug("处理消息时间[{}]", timer.interval());
        } catch (Exception e) {
            log.error("channelRead", e);
        }
    }

    public void handlerMsg(ChannelHandlerContext channelHandlerContext, Object msg) throws Exception {
        if (StringUtil.isJSONObject((String) msg)) {
            NettyMessage message = StringUtil.tranferItemToDTO((String) msg, NettyMessage.class);
            NettyMsgTypeEnum opCode = message.getOpCode();
            // 通过增加synchronized关键字控制同一类消息顺序执行，
            // 避免出现同时更新导致的异常。例如：先接到删除数据的信息，又接到新增数据的信息
            // 如果不保证执行顺序，很可能无法得到想要的结果
            switch (opCode) {
                case ALL_CONDITIONS:
                    synchronized ("sync_condition") {
                        nettyMsgHandler.cacheAllAlarmConditions(msg);
                    }
                    break;
                case NEW_CONDITION:
                    synchronized ("sync_condition") {
                        nettyMsgHandler.cacheNewCondition(msg);
                    }
                    break;
                case UPDATE_CONDITION:
                    synchronized ("sync_condition") {
                        nettyMsgHandler.cacheUpdatedCondition(msg);
                    }
                    break;
                case DELETE_CONDITION:
                    synchronized ("sync_condition") {
                        nettyMsgHandler.removeCachedCondition(msg);
                    }
                    break;
                case ALL_OBJ_CONDITION_REL:
                    synchronized ("sync_obj_condition_rel") {
                        nettyMsgHandler.cacheAllObjConditionRel(msg);
                    }
                    break;
                case NEW_OBJ_CONDITION_REL:
                    synchronized ("sync_obj_condition_rel") {
                        nettyMsgHandler.cacheNewObjConditionRel(msg);
                    }
                    break;
                case DELETE_OBJ_CONDITION_REL:
                    synchronized ("sync_obj_condition_rel") {
                        nettyMsgHandler.removeCachedObjConditionRel(msg);
                    }
                    break;
                case RECORD_ID:
                    nettyMsgHandler.cacheRecordId(message);
                    break;
                default:
                    break;
            }
            // 发送已接收回执
            nettyMsgHandler.acceptedReply(channelHandlerContext);
        }
    }

    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) {
        ctx.flush();
    }

    /**
     * 在处理过程中引发异常时被调用
     *
     * @param ctx
     * @param cause
     * @throws Exception
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        cause.printStackTrace();
        ctx.close();
    }

    /**
     * 通道处于非活跃状态动作，该方法只会在失效时调用一次
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        //客户端自己不正常情况下自己在重连一次
        log.info("Disconnected from: " + ctx.channel().remoteAddress());

    }

    @Override
    public void channelUnregistered(final ChannelHandlerContext ctx) throws Exception {
        log.info("Sleeping for: " + RECONNECT_DELAY + "s,Reconnecting to: " + groupNettyClient.host + ':' + groupNettyClient.port);
        ctx.channel().eventLoop().schedule(new Runnable() {
            @Override
            public void run() {
                log.info("Reconnecting to: " + groupNettyClient.host + ':' + groupNettyClient.port);
                groupNettyClient.connect(groupNettyClient, ctx.channel());
            }
        }, RECONNECT_DELAY, TimeUnit.SECONDS);
    }
}
